import 'dart:async';
import 'package:flutter/material.dart';
import 'package:nuntio_hera/hera_app/helpers.dart';
import 'package:nuntio_hera/hera_app/login_screen.dart';
import 'package:nuntio_core/nuntio_theme.dart';
import 'package:nuntio_core/models/hera.pb.dart';
import '../../hera/hera.dart';
import '../hera/models/auth.dart';
import 'hera_localizations.dart';

class HeraApp extends StatefulWidget {
  HeraApp({
    Key? key,
    required this.child,
    this.excludeAuthMechanisms,
    this.excludeOidcProviders,
    this.localizationsDelegates,
    this.logo,
  });

  final List<LoginType>? excludeAuthMechanisms;
  final List<OidcProviderType>? excludeOidcProviders;
  final Widget child;
  final Widget? logo;

  /// The delegates for this app's [Localizations] widget.
  ///
  /// The delegates collectively define all of the localized resources
  /// for this application's [Localizations] widget.
  ///
  /// Internationalized apps that require translations for one of the locales
  /// listed in [GlobalHeraLocalizations] should specify this parameter
  /// and list the [supportedLocales] that the application can handle.
  ///
  /// ```dart
  /// import 'package:flutter_localizations/flutter_localizations.dart';
  /// HeraApp(
  ///   localizationsDelegates: [
  ///     // ... app-specific localization delegate[s] here
  ///     GlobalHeraLocalizations.delegate,
  ///     GlobalMercuryLocalizations.delegate,
  ///   ],
  ///   supportedLocales: [
  ///     const Locale('en', 'US'), // English
  ///     const Locale('da', 'DK'), // Danish
  ///     // ... other locales the app supports
  ///   ],
  ///   // ...
  /// )
  /// ```
  ///
  /// ## Adding localizations for a new locale
  ///
  /// The information that follows applies to the unusual case of an app
  /// adding translations for a language not already supported by
  /// [GlobalHeraLocalizations].
  ///
  /// Delegates that produce [HeraLocalizations] and [MercuryLocalizations]
  /// are included automatically. Apps can provide their own versions of these
  /// localizations by creating implementations of
  /// [LocalizationsDelegate<HeraLocalizations>] or
  /// [LocalizationsDelegate<MercuryLocalizations>] whose load methods return
  /// custom versions of [HeraLocalizations] or [MercuryLocalizations].
  ///
  /// For example: to add support to [heraLocalizations] for a
  /// locale it doesn't already support, say `const Locale('foo', 'BR')`,
  /// one could just extend [DefaultHeraLocalizations]:
  ///
  /// ```dart
  /// class FooLocalizations extends DefaultHeraLocalizations {
  ///   FooLocalizations(Locale locale) : super(locale);
  ///   @override
  ///   String get okButtonLabel {
  ///     if (locale == const Locale('foo', 'BR'))
  ///       return 'foo';
  ///     return super.okButtonLabel;
  ///   }
  /// }
  ///
  /// ```
  ///
  /// A `FooLocalizationsDelegate` is essentially just a method that constructs
  /// a `FooLocalizations` object. We return a [SynchronousFuture] here because
  /// no asynchronous work takes place upon "loading" the localizations object.
  ///
  /// ```dart
  /// class FooLocalizationsDelegate extends LocalizationsDelegate<HeraLocalizations> {
  ///   const FooLocalizationsDelegate();
  ///   @override
  ///   Future<FooLocalizations> load(Locale locale) {
  ///     return SynchronousFuture(FooLocalizations(locale));
  ///   }
  ///   @override
  ///   bool shouldReload(FooLocalizationsDelegate old) => false;
  /// }
  /// ```
  ///
  /// Constructing a [HeraApp] with a `FooLocalizationsDelegate` overrides
  /// the automatically included delegate for [HeraLocalizations] because
  /// only the first delegate of each [LocalizationsDelegate.type] is used and
  /// the automatically included delegates are added to the end of the app's
  /// [localizationsDelegates] list.
  ///
  /// ```dart
  /// HeraApp(
  ///   localizationsDelegates: [
  ///     const FooLocalizationsDelegate(),
  ///   ],
  ///   // ...
  /// )
  /// ```
  /// See also:
  ///
  ///  * [supportedLocales], which must be specified along with
  ///    [localizationsDelegates].
  ///  * [GlobalHeraLocalizations], a [localizationsDelegates] value
  ///    which provides Hera localizations for many languages.
  ///  * The Flutter Internationalization Tutorial,
  ///    <https://flutter.dev/tutorials/internationalization/>.
  final Iterable<LocalizationsDelegate<dynamic>>? localizationsDelegates;

  // on authenticated go to child;
  @override
  State<HeraApp> createState() => _HeraAppState();
}

class _HeraAppState extends State<HeraApp> {
  late AuthState _authState;
  late Config _config;
  late TextInputType _inputType;

  Future<void> initialize() async {
    // get config used to login
    _config = await Hera.initializeAuth();
    if (await Hera.isAuthenticated()) {
      setState(() {
        _authState = AuthState.authenticated;
        _inputType = getInputType(_config.supportedLoginMechanisms);
      });
      return;
    }
    // do not display oidc proivers that are not enabled
    _config.oidcProviders.removeWhere((key, value) =>
        !value.enabled ||
        widget.excludeOidcProviders?.contains(value.type) == true);
    _config.supportedLoginMechanisms.removeWhere(
        (element) => widget.excludeAuthMechanisms?.contains(element) ?? false);
    setState(() {
      _inputType = getInputType(_config.supportedLoginMechanisms);
      _authState = AuthState.notAuthenticated;
    });
  }

  @override
  void initState() {
    Hera.onLogout = () {
      setState(() {
        _authState = AuthState.notAuthenticated;
      });
    };
    setState(() {
      _authState = AuthState.loading;
    });
    initialize();
    super.initState();
  }

  // Combine the Localizations for Hera with the ones contributed
  // by the localizationsDelegates parameter, if any. Only the first delegate
  // of a particular LocalizationsDelegate.type is loaded so the
  // localizationsDelegate parameter can be used to override
  // _MaterialLocalizationsDelegate.
  Iterable<LocalizationsDelegate<dynamic>> get _localizationsDelegates {
    return <LocalizationsDelegate<dynamic>>[
      if (widget.localizationsDelegates != null)
        ...widget.localizationsDelegates!,
      DefaultHeraLocalizations.delegate,
      //TODO: DefaultMerucuryLocalizations.delegate,
    ];
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      debugShowCheckedModeBanner: false,
      localizationsDelegates: _localizationsDelegates,
      theme: Theme.of(context).brightness == Brightness.light
          ? nuntioLightTheme
          : nuntioDarkTheme,
      home: Builder(
        builder: (context) {
          switch (_authState) {
            case AuthState.loading:
              return Center(
                child: CircularProgressIndicator(),
              );
            case AuthState.authenticated:
              return widget.child;
            case AuthState.notAuthenticated:
              return LoginScreen(
                inputType: _inputType,
                config: _config,
                onAuthenticated: (BuildContext context) {
                  initialize();
                },
              );
            default:
              return SizedBox(child: Text("error"));
          }
        },
      ),
    );
  }
}

class Test extends DefaultHeraLocalizations {
  @override
  String get next => "Forsæt";
}
